@@ -1,5 +1,15 @@
 from concurrent.futures import ThreadPoolExecutor
-from typing import Any, Collection, Iterable, List, Mapping, Optional, Sequence, Union
+from typing import (
+    Any,
+    Collection,
+    Dict,
+    Iterable,
+    List,
+    Mapping,
+    Optional,
+    Sequence,
+    Union,
+)
 
 try:
     import google.cloud.bigquery as bigquery
@@ -7,6 +17,12 @@ except ImportError:
     print("Please install metaphor[bigquery] extra\n")
     raise
 
+from metaphor.bigquery.config import BigQueryRunConfig
+from metaphor.bigquery.utils import build_client
+from metaphor.common.event_util import ENTITY_TYPES
+from metaphor.common.extractor import BaseExtractor
+from metaphor.common.filter import DatasetFilter
+from metaphor.common.logger import get_logger
 from metaphor.models.crawler_run_metadata import Platform
 from metaphor.models.metadata_change_event import (
     DataPlatform,
@@ -20,13 +36,6 @@ from metaphor.models.metadata_change_event import (
     SQLSchema,
 )
 
-from metaphor.bigquery.config import BigQueryRunConfig
-from metaphor.bigquery.utils import build_client
-from metaphor.common.event_util import ENTITY_TYPES
-from metaphor.common.extractor import BaseExtractor
-from metaphor.common.filter import DatasetFilter
-from metaphor.common.logger import get_logger
-
 logger = get_logger(__name__)
 
 
@@ -48,36 +57,50 @@ class BigQueryExtractor(BaseExtractor):
     async def extract(self, config: BigQueryRunConfig) -> Collection[ENTITY_TYPES]:
         assert isinstance(config, BigQueryExtractor.config_class())
 
-        logger.info("Fetching metadata from BigQuery")
+        logger.info(f"Fetching metadata from BigQuery project {config.project_id}")
 
         client = build_client(config)
         dataset_filter = config.filter.normalize()
 
-        with ThreadPoolExecutor(max_workers=config.max_concurrency) as executor:
+        fetched_tables: List[Dataset] = []
+        for dataset_ref in BigQueryExtractor._list_datasets_with_filter(
+            client, dataset_filter
+        ):
+            logger.info(f"Fetching tables for {dataset_ref}")
 
-            def list_table(dataset_ref) -> Iterable[Dataset]:
-                def get_table(table) -> Dataset:
+            with ThreadPoolExecutor(max_workers=config.max_concurrency) as executor:
+
+                def get_table(table: bigquery.TableReference) -> Dataset:
+                    logger.info(f"Getting table {table.table_id}")
                     bq_table = client.get_table(table)
                     return self._parse_table(client.project, bq_table)
 
-                return executor.map(
-                    get_table,
-                    BigQueryExtractor._list_tables_with_filter(
-                        dataset_ref, client, dataset_filter
-                    ),
-                )
+                # map of table name to Dataset
+                tables: Dict[str, Dataset] = {
+                    d.logical_id.name.split(".")[-1]: d
+                    for d in executor.map(
+                        get_table,
+                        BigQueryExtractor._list_tables_with_filter(
+                            dataset_ref, client, dataset_filter
+                        ),
+                    )
+                }
 
-            def flatten(result: Iterable[Iterable[Dataset]]) -> List[Dataset]:
-                return [d for datasets in result for d in datasets]
+            logger.info(f"Getting table DDL for {dataset_ref}")
+            table_ddl = client.query(
+                f"select table_name, ddl from `{dataset_ref.project}.{dataset_ref.dataset_id}.INFORMATION_SCHEMA.TABLES`"
+            ).result()
 
-            return flatten(
-                executor.map(
-                    list_table,
-                    BigQueryExtractor._list_datasets_with_filter(
-                        client, dataset_filter
-                    ),
-                )
-            )
+            for table_name, ddl in table_ddl:
+                table = tables[str(table_name).lower()]
+                if not table:
+                    logger.error(f"table {table_name} not found for DDL")
+                    continue
+                table.schema.sql_schema.table_schema = ddl
+
+            fetched_tables.extend(tables.values())
+
+        return fetched_tables
 
     @staticmethod
     def _list_datasets_with_filter(
@@ -91,8 +114,6 @@ class BigQueryExtractor(BaseExtractor):
             dataset_ref = bigquery.DatasetReference(
                 client.project, bq_dataset.dataset_id
             )
-
-            logger.info(f"Found dataset {dataset_ref}")
             yield dataset_ref
 
     @staticmethod
@@ -108,7 +129,6 @@ class BigQueryExtractor(BaseExtractor):
             ):
                 logger.info(f"Skipped table: {table_ref}")
                 continue
-            logger.info(f"Found table {table_ref}")
             yield table_ref
 
     # See https://googleapis.dev/python/bigquery/latest/generated/google.cloud.bigquery.table.Table.html#google.cloud.bigquery.table.Table
