@@ -1,4 +1,3 @@
-import json
 from datetime import datetime, timedelta, timezone
 from typing import Collection, Dict, Optional
 
@@ -8,14 +7,6 @@ except ImportError:
     print("Please install metaphor[bigquery] extra\n")
     raise
 
-from metaphor.models.crawler_run_metadata import Platform
-from metaphor.models.metadata_change_event import (
-    DataPlatform,
-    Dataset,
-    DatasetLogicalID,
-    DatasetUpstream,
-    EntityType,
-)
 from sql_metadata import Parser
 
 from metaphor.bigquery.lineage.config import BigQueryLineageRunConfig
@@ -26,6 +17,14 @@ from metaphor.common.event_util import ENTITY_TYPES
 from metaphor.common.extractor import BaseExtractor
 from metaphor.common.filter import DatasetFilter
 from metaphor.common.logger import get_logger
+from metaphor.models.crawler_run_metadata import Platform
+from metaphor.models.metadata_change_event import (
+    DataPlatform,
+    Dataset,
+    DatasetLogicalID,
+    DatasetUpstream,
+    EntityType,
+)
 
 logger = get_logger(__name__)
 
@@ -47,6 +46,7 @@ class BigQueryLineageExtractor(BaseExtractor):
         self._utc_now = datetime.now().replace(tzinfo=timezone.utc)
         self._datasets: Dict[str, Dataset] = {}
         self._dataset_filter: DatasetFilter = DatasetFilter()
+        self._include_self_lineage = True
 
     async def extract(
         self, config: BigQueryLineageRunConfig
@@ -137,7 +137,7 @@ class BigQueryLineageExtractor(BaseExtractor):
     def _fetch_audit_log(self, config: BigQueryLineageRunConfig):
         logger.info("Fetching lineage info from BigQuery Audit log")
 
-        client = build_logging_client(config.key_path, config.project_id)
+        client = build_logging_client(config)
         self._dataset_filter = config.filter.normalize()
 
         log_filter = self._build_job_change_filter(config, end_time=self._utc_now)
@@ -160,14 +160,19 @@ class BigQueryLineageExtractor(BaseExtractor):
 
     def _parse_job_change_entry(self, entry: LogEntry):
         job_change = JobChangeEvent.from_entry(entry)
-        if job_change is None:
+        if job_change is None or job_change.destination_table is None:
             return
 
-        if job_change.destination_table in job_change.source_tables:
-            logger.warning(
-                f"self referencing tables in job {json.dumps(job_change.__dict__, default=str)}"
+        assert job_change.destination_table is not None
+
+        # Filter out self lineage
+        if not self._include_self_lineage:
+            job_change.source_tables = list(
+                filter(
+                    lambda t: t != job_change.destination_table,
+                    job_change.source_tables,
+                )
             )
-            return
 
         destination = job_change.destination_table
         if not self._dataset_filter.include_schema(
