@@ -1,15 +1,6 @@
 from datetime import timedelta
 from typing import Collection, List, Optional, Set
 
-from metaphor.models.crawler_run_metadata import Platform
-from metaphor.models.metadata_change_event import (
-    DataPlatform,
-    Dataset,
-    DatasetLogicalID,
-    DatasetQueryHistory,
-    QueryInfo,
-)
-
 from metaphor.bigquery.logEvent import JobChangeEvent
 from metaphor.bigquery.query.config import BigQueryQueryRunConfig
 from metaphor.bigquery.utils import LogEntry, build_logging_client
@@ -19,6 +10,14 @@ from metaphor.common.filter import DatasetFilter
 from metaphor.common.logger import get_logger
 from metaphor.common.query_history import TableQueryHistoryHeap
 from metaphor.common.utils import start_of_day
+from metaphor.models.crawler_run_metadata import Platform
+from metaphor.models.metadata_change_event import (
+    DataPlatform,
+    Dataset,
+    DatasetLogicalID,
+    DatasetQueryHistory,
+    QueryInfo,
+)
 
 logger = get_logger(__name__)
 
@@ -47,7 +46,7 @@ class BigQueryQueryExtractor(BaseExtractor):
 
         logger.info("Fetching query history from BigQuery audit log")
 
-        client = build_logging_client(config.key_path, config.project_id)
+        client = build_logging_client(config)
         self._dataset_filter = config.filter.normalize()
         self._excluded_usernames = config.excluded_usernames
         self._table_queries = TableQueryHistoryHeap(config.max_queries_per_table)
@@ -93,11 +92,21 @@ class BigQueryQueryExtractor(BaseExtractor):
 
             table_name = queried_table.table_name()
 
+            if job_change.start_time and job_change.end_time:
+                elapsed_time = (
+                    job_change.end_time - job_change.start_time
+                ).total_seconds()
+            else:
+                elapsed_time = None
+
             self._table_queries.store_recent_query(
                 table_name,
-                job_change.timestamp,
-                job_change.query,
-                job_change.user_email,
+                QueryInfo(
+                    elapsed_time=elapsed_time,
+                    issued_at=job_change.timestamp,
+                    issued_by=job_change.user_email,
+                    query=job_change.query,
+                ),
             )
 
     def _init_dataset(
